#
# This is the server logic of a Shiny web application. You can run the 
# application by clicking 'Run App' above.
#
# Find out more about building applications with Shiny here:
# 
#    http://shiny.rstudio.com/
#

library(shiny)

# Define server logic required to draw a histogram
shinyServer(function(input, output) {
  
  file_name <- reactive({
    inFile <- input$Photolist
    if (is.null(inFile))
      return(NULL)
    
    return (stringi::stri_extract_first(str = inFile$name, regex = ".*(?=\\.)"))
  })
  
  file_location <- reactive({
    inFile <- input$Photolist
    if (is.null(inFile))
      return(NULL)
    return (inFile$datapath )
  })
  
  #output$myFileName <- renderText({ file_name() })
  #output$myFilePath <- renderText({ file_location() })
  
  #output$radio <- reactiveUI(function() {
  #  
  #  radioButtons("This_File", "Choose a photo to measure", 
  #               choices = c("None", file_name()))
  #  
  #})
  
  output$radio <- reactiveUI(function() {
    
    radioButtons("This_File", "Choose a photo to measure", 
                 choiceNames = c("None", file_name()), 
                 choiceValues = c("NA", file_location()))
    
  })
  
  output$myFilePath <- renderText({ input$This_File })
  
  output$Plot <- renderPlot({
    img <- readJPEG(input$This_File)
    op <- par(mar=c(0,0,0,0))
    on.exit(par(op))
    plot.new()
    plot <- rasterImage(img,0,0,1,1)
    return(plot)
    #}
   }) 
  

# observeEvent(input$measuredist, {
#    showModal(modalDialog(
#      title = "Scale identifier",
#      "Click on four points (x1, x2, y1, y2) for x1 and x2 are the scale bar on the horizontal, 
#                            and y1 and y2 is the scale bar on the vertical.",
#      easyClose = FALSE
#    ))
#  })
  
  #observeEvent(input$measuredist, {
  #  ReadAndCal(input$This_File)
  #  })
  
  output$resetable_input <- renderUI({
    times <- input$save
    div(id=letters[(times %% length(letters)) + 1],
    textInput("ID", "Photo ID"),
    textInput("leg", "Leg information",placeholder = "e.g. hind, front, left, right"),
     radioButtons("view", "View",
               c("Front" = "front",
                 "Side" = "side",
                 "Other" = "other"))
    )
  })

# set up input values for hoof  
   output$resetable_input_side <- renderUI({
     times <- input$save
      div(id=letters[(times %% length(letters)) + 1],
       numericInput('width', 'Enter width of the scale bar',0),
       numericInput('hight', 'Enter height of the scale bar',0),
       actionButton("plotpoints", label = "Click on horizontal and vertical scale bars and then click here once complete"),
       
      if(input$view == 'side') {
        list(
          textInput('growth', 'Growth Score', placeholder = "0,1,2,unknown (u)"),
          textInput('heel', 'Heel Score', placeholder = "0,1,2,unknown (u)" ),
          textInput('Fetlock', 'Fetlock integrity', placeholder = '0,1,unknown (u)'),
          textAreaInput('comments',  'Comments', rows = 2, cols = 80),
          actionButton("measureratio", "Click three points you wich to measure the ratio of two distance on plot then click here once complete")
        )
      },
      if(input$view == 'front'){
        list(
          textInput('shape', 'Shape Score (measure distance of splay only if shape = 0)', placeholder = "0,1,2,unknown (u)"),
          textInput('splay', 'Visual score of splay', placeholder = "0,1,2,unknown (u)" ),
          textAreaInput('comments',  'Comments', rows = 2, cols = 80),
          actionButton("measuredist", "Click splay distance on plot then click here once complete")
        )
      } 
    )
 })

## start recording clicks
xy_new <- reactiveValues(x= numeric(0), y = numeric(0), line=numeric(0)) # add new points



  # Listen for clicks and store values
  observe({
    if (is.null(input$plot_click)){
      return()
    }
    
    isolate({
      xy_new$x <- c(xy_new$x, input$plot_click$x)
      xy_new$y <- c(xy_new$y, input$plot_click$y)
    })
  })
  
  pointsforplot <- eventReactive(input$plotpoints, ignoreNULL = F, {
    data.points = as.data.frame(xy_str(xy_new))
    x = xy_new$x
    y = xy_new$y
    data.points = data.frame(x,y)
    #as.data.frame(x = xy_new$x, y = xy_new$y)
  })
  
  output$points <- renderDataTable({ pointsforplot() })
  
  distanceSpray <- eventReactive(input$measuredist, ignoreNULL = T, {
    cal = pointsforplot()
    nrows = dim(pointsforplot())[1]
    x = xy_new$x
    y = xy_new$y
    data.distance = data.frame(x,y)
    yrows = dim(data.distance)[1]
    data.points = data.distance[(nrows+1):yrows,]
    x1    <- 0
    x2    <- 0 + input$width
    y1    <- 0
    y2    <- 0 + input$hight
    x     <- cal$x[c(1,2)]
    y     <- cal$y[c(3,4)]
    cx <- lm(formula = c(x1,x2) ~ c(x))$coeff
    cy <- lm(formula = c(y1,y2) ~ c(y))$coeff

    data.points$x <- data.points$x*cx[2]+cx[1]
    data.points$y <- data.points$y*cy[2]+cy[1]

    data.points <- data.points[order(data.points$x),]
    nrow(data.points)
    if(nrow(data.points) == 2){
      distance = sqrt((data.points$x[2]- data.points$x[1])^2 +
                        (data.points$y[2]- data.points$y[1])^2)
    }else distance = NA

    distance
  })

  output$distance <- renderText({ distanceSpray() })
#
# # ratio of spray when side view.
  ratioSpray <- eventReactive(input$measureratio, ignoreNULL = T, {
    cal = pointsforplot()
    nrows = dim(pointsforplot())[1]
    x = xy_new$x
    y = xy_new$y
    data.distance = data.frame(x,y)
    yrows = dim(data.distance)[1]
    data.points = data.distance[(nrows+1):yrows,]
    x1    <- 0
    x2    <- 0 + input$width
    y1    <- 0
    y2    <- 0 + input$hight
    x     <- cal$x[c(1,2)]
    y     <- cal$y[c(3,4)]
    cx <- lm(formula = c(x1,x2) ~ c(x))$coeff
    cy <- lm(formula = c(y1,y2) ~ c(y))$coeff

    data.points$x <- data.points$x*cx[2]+cx[1]
    data.points$y <- data.points$y*cy[2]+cy[1]

    data.points <- data.points[order(data.points$x),]
    n <- nrow(data.points)
    if(n == 3){
      ratio = sqrt((data.points$x[2]- data.points$x[1])^2 + (data.points$y[2]- data.points$y[1])^2)/
        sqrt((data.points$x[2]- data.points$x[3])^2 + (data.points$y[2]- data.points$y[3])^2)
    }else ratio = NA

    ratio
  })

  output$ratio <- renderText({ ratioSpray() })

  resultsData <- data.frame(
    PhotoName = NA,
    PhotoID = NA,
    LegInfo = NA,
    Veiw = NA,
    Width = NA, 
    Height = NA, 
    Shape = NA, 
    Splay = NA, 
    distance = NA, 
    Growth = NA,  
    Heel = NA, 
    Fetlock.integrity = NA, 
    ratio = NA,
    Comments=NA)
   
 # saving the data into a data frame
  savedata <- reactive({
    results <- data.frame(
      PhotoName = NA,
      PhotoID = NA,
      LegInfo = NA,
      Veiw = NA,
      Width = NA, 
      Height = NA, 
      Shape = NA, 
      Splay = NA, 
      distance = NA, 
      Growth = NA,  
      Heel = NA, 
      Fetlock.integrity = NA, 
      ratio = NA,
      Comments=NA)
    if(input$view == 'front'){
    results <- data.frame(
      PhotoName = input$This_File,
      PhotoID = input$ID,
      LegInfo = input$leg,
      Veiw = input$view,
      Width = input$width, 
      Height = input$hight, 
      Shape = input$shape, 
      Splay = input$splay, 
      distance = distanceSpray(), 
      Growth = NA,  
      Heel = NA, 
      Fetlock.integrity = NA, 
      ratio = NA,
      Comments=input$comments)
    }
    if(input$view == 'side'){
      results <- data.frame(
        PhotoName = input$This_File,
        PhotoID = input$ID,
        LegInfo = input$leg,
        Veiw = input$view,
        Width = input$width, 
        Height = input$hight, 
        Shape = NA, 
        Splay = NA, 
        distance = NA, 
        Growth = input$growth,  
        Heel = input$heel, 
        Fetlock.integrity = input$Fetlock, 
        ratio = ratioSpray(),
        Comments=input$comments)
    }
    results
  })
    
    observeEvent(input$save, {
      resultsData <<- rbind(resultsData, savedata())
      output$table <- renderTable(resultsData)
      xy_new$x <- NULL
      xy_new$y <- NULL
      })
    
  
     output$downloadData <- downloadHandler(
      
      # Create the download file name
      filename = function() {
        paste("data-", Sys.Date(), ".csv", sep="")
      },
      content = function(file) {
        write.csv(resultsData, file)                     # put Data() into the download file
      })                          
    
    
  
  
})

### Functions
need<-c("jpeg","tcltk2","zoo", "lattice", "latticeExtra") #needed libraries
ins<-installed.packages()[,1] #find out which libs are installed
(Get<-need[which(is.na(match(need,ins)))])
if(length(Get)>0){install.packages(Get)} #install needed libs
eval(parse(text=paste("library(",need,")")))#load libraries

Calibrate = function(data,calpoints,Width, Height)
{
  x1    <- 0
  x2    <- 0 + as.numeric(Width)
  y1    <- 0
  y2    <- 0 + as.numeric(Height)
  x     <- calpoints$x[c(1,2)]
  y     <- calpoints$y[c(3,4)]
  
  cx <- lm(formula = c(x1,x2) ~ c(x))$coeff
  cy <- lm(formula = c(y1,y2) ~ c(y))$coeff
  
  data$x <- data$x*cx[2]+cx[1]
  data$y <- data$y*cy[2]+cy[1]
  
  return(as.data.frame(data))
}

xy_str <- function(e) {
  if(is.null(e)) return("NULL\n")
  paste0("x=", round(e$x, 2), " y=", round(e$y, 2), "\n")
}


